@@ -1,19 +1,17 @@
 /* Copyright (C) 2010 The cairomm Development Team
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __CAIROMM_DEVICE_H
@@ -86,6 +84,43 @@ public:
     RefPtr<Device> m_device;
   };
 
+
+  /**
+   * @since 1.10
+   */
+  enum class DeviceType
+  {
+      /**
+       *
+       */
+      DRM = CAIRO_DEVICE_TYPE_DRM,
+
+      /**
+       *
+       */
+      GL = CAIRO_DEVICE_TYPE_GL,
+
+      /**
+       *
+       */
+      SCRIPT = CAIRO_DEVICE_TYPE_SCRIPT,
+
+      /**
+       *
+       */
+      XCB = CAIRO_DEVICE_TYPE_XCB,
+
+      /**
+       *
+       */
+      XLIB = CAIRO_DEVICE_TYPE_XLIB,
+
+      /**
+       *
+       */
+      XML = CAIRO_DEVICE_TYPE_XML
+  };
+
   /** Create a C++ wrapper for the C instance. This C++ instance should then be given to a RefPtr.
    * @param cobject The C instance.
    * @param has_reference Whether we already have a reference. Otherwise, the constructor will take an extra reference.
