@@ -1,19 +1,17 @@
 /* Copyright (C) 2005 The cairomm Development Team
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __CAIROMM_FONTFACE_H
@@ -37,6 +35,10 @@ namespace Cairo
 class CAIROMM_API ScaledFont;
 class CAIROMM_API Context;
 
+ /** @example text-rotate.cc
+  * A simple example of using text in cairomm
+  */
+
 /**
  * A FontFace represents a particular font at a particular weight, slant, and
  * other characteristic but no size, transformation, or size.
@@ -105,6 +107,43 @@ class CAIROMM_API ToyFontFace : public FontFace
 {
 public:
 
+  /**
+   * Specifies variants of a font face based on their slant.
+   */
+  enum class Slant
+  {
+      /**
+       * Upright font style
+       */
+      NORMAL = CAIRO_FONT_SLANT_NORMAL,
+
+      /**
+       * Italic font style
+       */
+      ITALIC = CAIRO_FONT_SLANT_ITALIC,
+
+      /**
+       * Oblique font style
+       */
+      OBLIQUE = CAIRO_FONT_SLANT_OBLIQUE
+  };
+
+  /**
+   * Specifies variants of a font face based on their weight.
+   */
+  enum class Weight
+  {
+      /**
+       * Normal font weight
+       */
+      NORMAL = CAIRO_FONT_WEIGHT_NORMAL,
+
+      /**
+       * Bold font weight
+       */
+      BOLD = CAIRO_FONT_WEIGHT_BOLD
+  };
+
   /**
    * Creates a font face from a triplet of family, slant, and weight. These font
    * faces are used in implementation of the the Context "toy" font API.
@@ -120,7 +159,7 @@ public:
    * @param slant the slant for the font.
    * @param weight the weight for the font.
    */
-  static RefPtr<ToyFontFace> create(const std::string& family, FontSlant slant, FontWeight weight);
+  static RefPtr<ToyFontFace> create(const std::string& family, Slant slant, Weight weight);
 
   /**
    * Gets the familly name of a toy font.
@@ -130,15 +169,15 @@ public:
   /**
    * Gets the slant a toy font.
    */
-  FontSlant get_slant() const;
+  Slant get_slant() const;
 
   /**
    * Gets the weight a toy font.
    */
-  FontWeight get_weight() const;
+  Weight get_weight() const;
 
 protected:
-  ToyFontFace(const std::string& family, FontSlant slant, FontWeight weight);
+  ToyFontFace(const std::string& family, Slant slant, Weight weight);
 };
 
 
@@ -164,7 +203,7 @@ protected:
  *
  * public:
  *   static Cairo::RefPtr<MyUserFont> create() {
- *     return Cairo::RefPtr<MyUserFont>(new MyUserFont);
+ *     return Cairo::make_refptr_for_instance<MyUserFont>(new MyUserFont);
  *
  * protected:
  *   // implement render_glyph() and any other virtual functions you want to override
