@@ -1,19 +1,17 @@
 /* Copyright (C) 2005 The cairomm Development Team
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __CAIROMM_SURFACE_H
@@ -22,17 +20,7 @@
 #include <string>
 #include <vector>
 
-/* following is required for OS X */
-
-#ifdef nil
-#undef nil
 #include <sigc++/slot.h>
-#define nil NULL
-#else
-#include <sigc++/slot.h>
-#endif
-
-/* end OS X */
 
 #include <cairomm/enums.h>
 #include <cairomm/exception.h>
@@ -62,6 +50,8 @@
 
 namespace Cairo
 {
+class CAIROMM_API ImageSurface;
+class CAIROMM_API MappedImageSurface;
 
 /** A cairo surface represents an image, either as the destination of a drawing
  * operation or as source when drawing onto another surface. There are
@@ -78,6 +68,208 @@ namespace Cairo
 class CAIROMM_API Surface
 {
 public:
+  /**
+   * %Cairo::Surface::Type is used to describe the type of a given surface. The
+   * surface types are also known as "backends" or "surface backends" within
+   * cairo.
+   *
+   * The surface type can be queried with Surface::get_type()
+   *
+   * The various Cairo::Surface functions can be used with surfaces of
+   * any type, but some backends also provide type-specific functions
+   * that must only be called with a surface of the appropriate
+   * type.
+   *
+   * New entries may be added in future versions.
+   *
+   * @since 1.2
+   **/
+  enum class Type
+  {
+      /**
+       * The surface is of type image
+       */
+      IMAGE = CAIRO_SURFACE_TYPE_IMAGE,
+
+      /**
+       * The surface is of type pdf
+       */
+      PDF = CAIRO_SURFACE_TYPE_PDF,
+
+      /**
+       * The surface is of type ps
+       */
+      PS = CAIRO_SURFACE_TYPE_PS,
+
+      /**
+       * The surface is of type xlim
+       */
+      XLIB = CAIRO_SURFACE_TYPE_XLIB,
+
+      /**
+       * The surface is of type xcb
+       */
+      XCB = CAIRO_SURFACE_TYPE_XCB,
+
+      /**
+       * The surface is of type glitz
+       */
+      GLITZ = CAIRO_SURFACE_TYPE_GLITZ,
+
+      /**
+       * The surface is of type quartz
+       */
+      QUARTZ = CAIRO_SURFACE_TYPE_QUARTZ,
+
+#if !defined(WIN32) && !defined(CAIROMM_DISABLE_DEPRECATED)
+      /**
+       * The surface is of type win32
+       *
+       * @deprecated Use WIN32_SURFACE instead.
+       */
+      WIN32 = CAIRO_SURFACE_TYPE_WIN32,
+#endif // !WIN32 && !CAIROMM_DISABLE_DEPRECATED
+      /**
+       * The surface is of type win32
+       * @since 1.16.1
+       */
+      WIN32_SURFACE = CAIRO_SURFACE_TYPE_WIN32,
+
+      /**
+       * The surface is of type beos
+       */
+      BEOS = CAIRO_SURFACE_TYPE_BEOS,
+
+      /**
+       * The surface is of type directfb
+       */
+      DIRECTFB = CAIRO_SURFACE_TYPE_DIRECTFB,
+
+      /**
+       * The surface is of type svg
+       */
+      SVG = CAIRO_SURFACE_TYPE_SVG,
+
+      /**
+       * The surface is of type os2
+       */
+      OS2 = CAIRO_SURFACE_TYPE_OS2,
+
+      /**
+       * The surface is a win32 printing surface
+       */
+      WIN32_PRINTING = CAIRO_SURFACE_TYPE_WIN32_PRINTING,
+
+      /**
+       * The surface is of type quartz_image
+       */
+      QUARTZ_IMAGE = CAIRO_SURFACE_TYPE_QUARTZ_IMAGE,
+
+      /**
+       * The surface is of type script
+       * @since 1.10
+       */
+      SCRIPT = CAIRO_SURFACE_TYPE_SCRIPT,
+
+      /**
+       * The surface is of type Qt
+       * @since 1.10
+       */
+      QT = CAIRO_SURFACE_TYPE_QT,
+
+      /**
+       * The surface is of type recording
+       * @since 1.10
+       */
+      RECORDING = CAIRO_SURFACE_TYPE_RECORDING,
+
+      /**
+       * The surface is a OpenVG surface
+       * @since 1.10
+       */
+      VG = CAIRO_SURFACE_TYPE_VG,
+
+      /**
+       * The surface is of type OpenGL
+       * @since 1.10
+       */
+      GL = CAIRO_SURFACE_TYPE_GL,
+
+      /**
+       * The surface is of type Direct Render Manager
+       * @since 1.10
+       */
+      DRM = CAIRO_SURFACE_TYPE_DRM,
+
+      /**
+       * The surface is of type script 'tee' (a multiplexing surface)
+       * @since 1.10
+       */
+      TEE = CAIRO_SURFACE_TYPE_TEE,
+
+      /**
+       * The surface is of type XML (for debugging)
+       * @since 1.10
+       */
+      XML = CAIRO_SURFACE_TYPE_XML,
+
+      /**
+       * The surface is of type Skia
+       * @since 1.10
+       */
+      SKIA = CAIRO_SURFACE_TYPE_SKIA,
+
+      /**
+       * The surface is of type The surface is a subsurface created with
+       * Surface::create()
+       * @since 1.10
+       */
+      SUBSURFACE = CAIRO_SURFACE_TYPE_SUBSURFACE
+  };
+
+  //TODO: Add new formats (at least 3) when we can add API. See cairo/src/cairo.h.
+  /**
+   * %Cairo::Surface::Format is used to identify the memory format of image data.
+   *
+   * New entries may be added in future versions.
+   **/
+  enum class Format
+  {
+      /**
+       * Each pixel is a 32-bit quantity, with alpha in the upper 8 bits, then
+       * red, then green, then blue. The 32-bit quantities are stored
+       * native-endian. Pre-multiplied alpha is used. (That is, 50% transparent
+       * red is 0x80800000, not 0x80ff0000.)
+       */
+      ARGB32 = CAIRO_FORMAT_ARGB32,
+
+      /**
+       * Each pixel is a 32-bit quantity, with the upper 8 bits unused. Red,
+       * Green, and Blue are stored in the remaining 24 bits in that order.
+       */
+      RGB24 = CAIRO_FORMAT_RGB24,
+
+      /**
+       * Each pixel is a 8-bit quantity holding an alpha value
+       */
+      A8 = CAIRO_FORMAT_A8,
+
+      /**
+       * Each pixel is a 1-bit quentity holding an alpha value. Pixels are packed
+       * together into 32-bit quantities. The ordering of the bits matches the
+       * endianness of the platform. On a big-endian machine, the first pixel is in
+       * the uppermost bit, on a little endian machine the first pixel is in the
+       * least-significant bit.
+       */
+      A1 = CAIRO_FORMAT_A1,
+
+      /**
+       * Each pixel is a 16-bit quantity with red in the upper 5 bits, then green
+       * in the middle 6 bits, and blue in the lower 5 bits
+       */
+      RGB16_565 = CAIRO_FORMAT_RGB16_565
+  };
+
   /** For example:
    * <code>
    * ErrorStatus my_write_func(unsigned char* data, unsigned int length);
@@ -92,7 +284,7 @@ public:
    * @param length the amount of data to write
    * @return the status code of the write operation
    */
-  typedef sigc::slot<ErrorStatus, const unsigned char* /*data*/, unsigned int /*length*/> SlotWriteFunc;
+  typedef sigc::slot<ErrorStatus(const unsigned char* /*data*/, unsigned int /*length*/)> SlotWriteFunc;
 
   /**
    * This is the type of function which is called when a backend needs to read
@@ -105,7 +297,7 @@ public:
    * @param length the amount of data to read
    * @return the status code of the read operation
    */
-  typedef sigc::slot<ErrorStatus, unsigned char* /*data*/, unsigned int /*length*/> SlotReadFunc;
+  typedef sigc::slot<ErrorStatus(unsigned char* /*data*/, unsigned int /*length*/)> SlotReadFunc;
 
   /** Create a C++ wrapper for the C instance. This C++ instance should then be
    * given to a RefPtr.
@@ -137,7 +329,7 @@ public:
   /** For instance,
    * void on_destroy();
    */
-  typedef sigc::slot<void> SlotDestroy;
+  typedef sigc::slot<void()> SlotDestroy;
 
   /** Attach an image in the format mime_type to surface. To remove the data from
    * a surface, call unset_mime_data() with same mime type.
@@ -171,7 +363,7 @@ public:
   /** Retrieves the default font rendering options for the surface. This allows
    * display surfaces to report the correct subpixel order for rendering on
    * them, print surfaces to disable hinting of metrics and so forth. The
-   * result can then be used with cairo_scaled_font_create().
+   * result can then be used with Cairo::ScaledFont::create().
    *
    * @param options 	a FontOptions object into which to store the retrieved
    * options. All existing values are overwritten
@@ -222,10 +414,10 @@ public:
 
   /** Sets an offset that is added to the device coordinates determined by the
    * CTM when drawing to surface. One use case for this function is when we
-   * want to create a Surface that redirects drawing for a portion of
+   * want to create a %Surface that redirects drawing for a portion of
    * an onscreen surface to an offscreen surface in a way that is completely
    * invisible to the user of the cairo API. Setting a transformation via
-   * cairo_translate() isn't sufficient to do this, since functions like
+   * Cairo::Context::translate() isn't sufficient to do this, since functions like
    * Cairo::Context::device_to_user() will expose the hidden offset.
    *
    * Note that the offset only affects drawing to the surface, not using the
@@ -240,6 +432,45 @@ public:
    */
   void get_device_offset(double& x_offset, double& y_offset) const;
 
+  /** Sets a scale that is multiplied to the device coordinates determined by
+   * the CTM when drawing to surface. One common use for this is to render to
+   * very high resolution display devices at a scale factor, so that code that
+   * assumes 1 pixel will be a certain size will still work. Setting a
+   * transformation via Cairo::Context::scale() isn't sufficient to do this, since
+   * functions like Cairo::Context::device_to_user() will expose the hidden scale.
+   *
+   * Note that the scale affects drawing to the surface as well as using the
+   * surface in a source pattern.
+   *
+   * @param x_scale 	a scale factor in the X direction
+   * @param y_scale 	a scale factor in the Y direction
+   *
+   * @newin{1,18}
+   */
+  void set_device_scale(double x_scale, double y_scale);
+
+  /** Sets x and y scale to the same value.
+   * See set_device_scale(double, double) for details.
+   *
+   * @param scale 	a scale factor in the X and Y direction
+   *
+   * @newin{1,18}
+   */
+  inline void set_device_scale(double scale) { set_device_scale(scale, scale); }
+
+  /** Returns a previous device scale set by set_device_scale().
+   *
+   * @newin{1,18}
+   */
+  void get_device_scale(double& x_scale, double& y_scale) const;
+
+  /** Returns the x and y average of a previous device scale set by
+   * set_device_scale().
+   *
+   * @newin{1,18}
+   */
+  double get_device_scale() const;
+
   /**
    * Set the horizontal and vertical resolution for image fallbacks.
    *
@@ -281,7 +512,7 @@ public:
    */
   void get_fallback_resolution(double& x_pixels_per_inch, double& y_pixels_per_inch) const;
 
-  SurfaceType get_type() const;
+  Type get_type() const;
 
   /**
    * This function returns the content type of surface which indicates whether
@@ -346,11 +577,69 @@ public:
    */
   void write_to_png_stream(const SlotWriteFunc& write_func);
 
-  /** @deprecated Use write_to_png_stream instead */
-  void write_to_png(cairo_write_func_t write_func, void *closure);
-
 #endif // CAIRO_HAS_PNG_FUNCTIONS
 
+  /** Create a new image surface that is as compatible as possible for uploading
+   * to and the use in conjunction with an existing surface.
+   * However, this surface can still be used like any normal image surface.
+   * Unlike Surface::create(const RefPtr<Surface> other, Content content,
+   * int width, int height) the new image surface won't inherit the device scale
+   * from this surface..
+   *
+   * Initially the surface contents are all 0 (transparent if contents have
+   * transparency, black otherwise.)
+   *
+   * Use Surface::create(const RefPtr<Surface> other, Content content,
+   * int width, int height) if you don't need an image surface.
+   *
+   * @param format The format for the new surface.
+   * @param width Width of the new surface (in pixels).
+   * @param height Height of the new surface (in pixels).
+   *
+   * @throws std::bad_alloc, Cairo::logic_error, std::ios_base::failure
+   * @newin{1,20}
+   */
+  RefPtr<ImageSurface> create_similar_image(Format format, int width, int height);
+
+  /** Checks whether @a mime_type is supported by the surface.
+   *
+   * @param mime_type The mime type.
+   *
+   * @newin{1,20}
+   */
+  bool supports_mime_type(const std::string& mime_type);
+
+  /** Returns an image surface that is the most efficient mechanism for
+   * modifying the backing store of the target surface.
+   *
+   * @note The use of the original surface as a target or source whilst it is
+   * mapped is undefined. The result of mapping the surface multiple times is
+   * undefined. Changing the device transform of the image surface or
+   * of this surface before the image surface is deleted results in
+   * undefined behavior.
+   *
+   * @param extents Limit the extraction to a rectangular region.
+   *
+   * @throws std::bad_alloc, Cairo::logic_error, std::ios_base::failure
+   * @newin{1,20}
+   */
+  RefPtr<MappedImageSurface> map_to_image(const RectangleInt& extents);
+
+  /** Returns an image surface that is the most efficient mechanism for
+   * modifying the backing store of the target surface.
+   * The region retrieved is the whole surface.
+   *
+   * @note The use of the original surface as a target or source whilst it is
+   * mapped is undefined. The result of mapping the surface multiple times is
+   * undefined. Changing the device transform of the image surface or
+   * of this surface before the image surface is deleted results in
+   * undefined behavior.
+   *
+   * @throws std::bad_alloc, Cairo::logic_error, std::ios_base::failure
+   * @newin{1,20}
+   */
+  RefPtr<MappedImageSurface> map_to_image();
+
   /** This function returns the device for a surface
    * @return The device for this surface, or an empty RefPtr if the surface has
    * no associated device */
@@ -414,7 +703,8 @@ protected:
   /** The underlying C cairo surface type that is wrapped by this Surface
    */
   cobject* m_cobject;
-};
+
+}; // end class Surface
 
 /** @example image-surface.cc
  * An example of using Cairo::ImageSurface class to render to PNG
@@ -422,19 +712,19 @@ protected:
 
 /** Image surfaces provide the ability to render to memory buffers either
  * allocated by cairo or by the calling code. The supported image formats are
- * those defined in Cairo::Format
+ * those defined in Format
  *
- * An ImageSurface is the most generic type of Surface and the only one that is
- * available by default.  You can either create an ImageSurface whose data is
- * managed by Cairo, or you can create an ImageSurface with a data buffer that
+ * An %ImageSurface is the most generic type of Surface and the only one that is
+ * available by default.  You can either create an %ImageSurface whose data is
+ * managed by Cairo, or you can create an %ImageSurface with a data buffer that
  * you allocated yourself so that you can have full access to the data.
  *
- * When you create an ImageSurface with your own data buffer, you are free to
+ * When you create an %ImageSurface with your own data buffer, you are free to
  * examine the results at any point and do whatever you want with it.  Note that
  * if you modify anything and later want to continue to draw to the surface
  * with cairo, you must let cairo know via Cairo::Surface::mark_dirty()
  *
- * Note that like all surfaces, an ImageSurface is a reference-counted object that should be used via Cairo::RefPtr.
+ * Note that like all surfaces, an %ImageSurface is a reference-counted object that should be used via Cairo::RefPtr.
  */
 class CAIROMM_API ImageSurface : public Surface
 {
@@ -505,14 +795,14 @@ public:
    * surface = Cairo::ImageSurface::create (data, format, width, height);
    * @endcode
    *
-   * @param format A Cairo::Format value
+   * @param format A Format value
    * @param width The desired width of an image surface to be created.
    * @return the appropriate stride to use given the desired format and width, or
    * -1 if either the format is invalid or the width too large.
    *
    * @since 1.6
    **/
-  static int format_stride_for_width(Cairo::Format format, int width);
+  static int format_stride_for_width(Format format, int width);
 
   /**
    * Creates an image surface of the specified format and dimensions. Initially
@@ -546,7 +836,9 @@ public:
    * @param format the format of pixels in the buffer
    * @param width the width of the image to be stored in the buffer
    * @param height the height of the image to be stored in the buffer
-   * @param stride the number of bytes between the start of rows in the buffer as allocated. This value should always be computed by cairo_format_stride_for_width() before allocating the data buffer.
+   * @param stride the number of bytes between the start of rows in the buffer as allocated.
+   *        This value should always be computed by format_stride_for_width()
+   *        before allocating the data buffer.
    * @return a RefPtr to the newly created surface.
    */
   static RefPtr<ImageSurface> create(unsigned char* data, Format format, int width, int height, int stride);
@@ -576,11 +868,49 @@ public:
    * contents of the PNG image file.
    */
   static RefPtr<ImageSurface> create_from_png_stream(const SlotReadFunc& read_func);
-  /** @deprecated Use create_from_png_stream instead */
-  static RefPtr<ImageSurface> create_from_png(cairo_read_func_t read_func, void *closure);
 
 #endif // CAIRO_HAS_PNG_FUNCTIONS
 
+}; // end class ImageSurface
+
+/** @example mapped-surface.cc
+ * An example of using Cairo::MappedImageSurface class to render to PDF
+ */
+
+/** Image surface which is mapped to a target surface.
+ *
+ * @see Surface::map_to_image()
+ * @newin{1,20}
+ */
+class CAIROMM_API MappedImageSurface : public ImageSurface
+{
+public:
+  /** Create a C++ wrapper for the C instance. This C++ instance should then be
+   * given to a RefPtr.
+   *
+   * @param cobject The C instance.
+   * @param ctarget The surface that has been mapped to this image surface.
+   * @param has_reference Whether we already have a reference. Otherwise, the
+   * constructor will take an extra reference.
+   *
+   * @newin{1,20}
+   */
+  MappedImageSurface(cairo_surface_t* cobject, cairo_surface_t* ctarget,
+    bool has_reference = false);
+
+  /** Destructor.
+   *
+   * Unmaps this image surface from the target surface that created it.
+   * The contents of the image will be uploaded to the target surface.
+   *
+   * @newin{1,20}
+   */
+  ~MappedImageSurface() override;
+
+protected:
+  /** The C cairo surface which has been mapped to this image surface.
+   */
+  cobject* m_ctarget;
 };
 
 
@@ -609,7 +939,7 @@ public:
  * necessary objects (paths, patterns, etc.), in order to achieve accurate
  * replay.
  *
- * Note that like all surfaces, a RecordingSurface is a reference-counted object
+ * Note that like all surfaces, a %RecordingSurface is a reference-counted object
  * that should be used via Cairo::RefPtr.
  */
 class CAIROMM_API RecordingSurface : public Surface
@@ -697,7 +1027,7 @@ typedef enum
   PDF_VERSION_1_5 = CAIRO_PDF_VERSION_1_5
 } PdfVersion;
 
-/** A PdfSurface provides a way to render PDF documents from cairo.  This
+/** A %PdfSurface provides a way to render PDF documents from cairo.  This
  * surface is not rendered to the screen but instead renders the drawing to a
  * PDF file on disk.
  *
@@ -738,8 +1068,6 @@ public:
    * @since 1.8
    */
   static RefPtr<PdfSurface> create_for_stream(const SlotWriteFunc& write_func, double width_in_points, double height_in_points);
-  /** @deprecated use PdfSurface::create_for_stream() instead */
-  static RefPtr<PdfSurface> create(cairo_write_func_t write_func, void *closure, double width_in_points, double height_in_points);
 
 /**
  * Changes the size of a PDF surface for the current (and subsequent) pages.
@@ -800,7 +1128,7 @@ typedef enum {
     PS_LEVEL_3 = CAIRO_PS_LEVEL_3
 } PsLevel;
 
-/** A PsSurface provides a way to render PostScript documents from cairo.  This
+/** A %PsSurface provides a way to render PostScript documents from cairo.  This
  * surface is not rendered to the screen but instead renders the drawing to a
  * PostScript file on disk.
  *
@@ -841,8 +1169,6 @@ public:
    * @since 1.8
    */
   static RefPtr<PsSurface> create_for_stream(const SlotWriteFunc& write_func, double width_in_points, double height_in_points);
-  /** @deprecated use PsSurface::create_for_stream() instead */
-  static RefPtr<PsSurface> create(cairo_write_func_t write_func, void *closure, double width_in_points, double height_in_points);
 
   /**
    * Changes the size of a PostScript surface for the current (and
@@ -955,7 +1281,7 @@ typedef enum
   SVG_VERSION_1_2 = CAIRO_SVG_VERSION_1_2
 } SvgVersion;
 
-/** A SvgSurface provides a way to render Scalable Vector Graphics (SVG) images
+/** A %SvgSurface provides a way to render Scalable Vector Graphics (SVG) images
  * from cairo.  This surface is not rendered to the screen but instead renders
  * the drawing to an SVG file on disk.
  *
@@ -998,9 +1324,6 @@ public:
    */
   static RefPtr<SvgSurface> create_for_stream(const SlotWriteFunc& write_func, double width_in_points, double height_in_points);
 
-  /** @deprecated Use SvgSurface::create_for_stream() instead */
-  static RefPtr<SvgSurface> create(cairo_write_func_t write_func, void *closure, double width_in_points, double height_in_points);
-
   /**
    * Restricts the generated SVG file to the given version. See get_versions()
    * for a list of available version values that can be used here.
@@ -1038,16 +1361,16 @@ public:
 
 #ifdef CAIRO_HAS_GLITZ_SURFACE
 
-/** A GlitzSurface provides a way to render to the X Window System using Glitz.
+/** A %GlitzSurface provides a way to render to the X Window System using Glitz.
  * This provides a way to use OpenGL-accelerated graphics from cairo.  If you
  * want to use hardware-accelerated graphics within the X Window system, you
  * should use this Surface type.
  *
  * @note For this Surface to be available, cairo must have been compiled with
- * Glitz support
+ * Glitz support.
  *
- * @warning This is an experimental surface.  It is not yet marked as a fully
- * supported surface by the cairo library
+ * @warning This was an experimental surface. Glitz support has been removed
+ * from cairo 1.18.
  */
 class CAIROMM_API GlitzSurface : public Surface
 {
