@@ -1,19 +1,17 @@
 /* Copyright (C) 2005 The cairomm Development Team
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __CAIROMM_WIN32_SURFACE_H
@@ -40,7 +38,7 @@ namespace Cairo
  * @note For this Surface to be available, cairo must have been compiled with
  * Win32 support
  */
-class Win32Surface : public Surface
+class CAIROMM_API Win32Surface : public Surface
 {
 public:
 
@@ -84,10 +82,6 @@ public:
    */
   static RefPtr<Win32Surface> create(HDC hdc);
 
-  /**
-   * @deprecated use create_with_dib()
-   */
-  static RefPtr<Win32Surface> create(Format format, int width, int height);
   /** Creates a device-independent-bitmap surface not associated with any
    * particular existing surface or device context. The created bitmap will be
    * unititialized.
@@ -124,7 +118,7 @@ public:
  *
  * @since 1.8
  */
-class Win32PrintingSurface : public Surface
+class CAIROMM_API Win32PrintingSurface : public Surface
 {
 public:
   explicit Win32PrintingSurface(cairo_surface_t* cobject, bool has_reference = false);
